==> reverie_dreams-0.4.7%2B1.21.7%2Bbeta-sources.jar: cc/thonly/reverie_dreams/datagen/JsonElementWriterProvider.java <==
package cc.thonly.reverie_dreams.datagen;

import cc.thonly.reverie_dreams.registry.content.block.RDBlocks;
import cc.thonly.reverie_dreams.block.base.FruitLeavesBlock;
import cc.thonly.reverie_dreams.datagen.generator.AbstractJsonElementWriterProvider;
import cc.thonly.reverie_dreams.data.FumoType;
import cc.thonly.reverie_dreams.registry.content.FumoTypes;
import cc.thonly.reverie_dreams.registry.content.block.RDWoodBlocks;
import cc.thonly.reverie_dreams.state.SixteenDirection;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import net.fabricmc.fabric.api.datagen.v1.FabricDataOutput;
import net.minecraft.class_2248;
import net.minecraft.class_2397;
import net.minecraft.class_2960;
import net.minecraft.class_7225;
import net.minecraft.class_7923;
import java.util.concurrent.CompletableFuture;

public class JsonElementWriterProvider extends AbstractJsonElementWriterProvider {
    public JsonElementWriterProvider(FabricDataOutput output, CompletableFuture<class_7225.class_7874> future) {
        super(output, future);
    }

    @Override
    protected void configured() {
        for (FumoType fumoType : FumoTypes.getView()) {
            class_2248 block = fumoType.block();
            this.addSixteenDirectionBlockState(block);
        }
        this.addSixteenDirectionBlockState(RDBlocks.MARISA_HAT_BLOCK);
        this.addFruityLeavesBlockState(RDWoodBlocks.LEMON_FRUIT_LEAVES, RDWoodBlocks.LEMON.leaves());
        this.addFruityLeavesBlockState(RDWoodBlocks.GINKGO_FRUIT_LEAVES, RDWoodBlocks.GINKGO.leaves());
        this.addFruityLeavesBlockState(RDWoodBlocks.PEACH_FRUIT_LEAVES, RDWoodBlocks.PEACH.leaves());
    }

    void addSixteenDirectionBlockState(class_2248 block) {
        class_2960 key = class_7923.field_41175.method_10221(block);
        JsonElement blockState = this.buildSixteenDirectionBlockState(block);
        JsonElement items = this.buildBlockItem(block);
        this.addElement(Type.ASSETS, key, "blockstates", blockState);
        this.addElement(Type.ASSETS, key, "items", items);
    }

    void addFruityLeavesBlockState(class_2248 block, class_2248 rawBlock) {
        class_2960 key = class_7923.field_41175.method_10221(block);
        JsonElement blockState = this.buildFruityLeavesBlock(block, rawBlock);
        JsonElement items = this.buildBlockItem(block);
        this.addElement(Type.ASSETS, key, "blockstates", blockState);
        this.addElement(Type.ASSETS, key, "items", items);
    }

    JsonElement buildFruityLeavesBlock(class_2248 block, class_2248 rawBlock) {
        class_2960 blockKey = class_7923.field_41175.method_10221(block);
        class_2960 rawBlockKey = class_7923.field_41175.method_10221(rawBlock);
        JsonObject object = new JsonObject();
        JsonObject variants = new JsonObject();
        for (int i = 0; i <= FruitLeavesBlock.MAX_AGE; i++) {
            for (Integer distance : class_2397.field_11199.method_11898()) {
                for (Boolean persistent : class_2397.field_11200.method_11898()) {
                    for (Boolean waterlogged : class_2397.field_38227.method_11898()) {
                        JsonObject element = new JsonObject();
                        String modelId = i == FruitLeavesBlock.MAX_AGE ? "%s:block/%s".formatted(blockKey.method_12836(), blockKey.method_12832()) : "%s:block/%s".formatted(rawBlockKey.method_12836(), rawBlockKey.method_12832());
                        element.addProperty("model", modelId);
                        String keyName = "fruit_age=%s,distance=%s,persistent=%s,waterlogged=%s".formatted(i, distance, persistent, waterlogged);
                        variants.add(keyName, element);
                    }
                }

            }
        }
        object.add("variants", variants);
        return object;
    }

    JsonElement buildSixteenDirectionBlockState(class_2248 block) {
        class_2960 key = class_7923.field_41175.method_10221(block);
        JsonObject object = new JsonObject();
        JsonObject variants = new JsonObject();

        for (SixteenDirection direction : SixteenDirection.values()) {
            JsonObject element = new JsonObject();
            float y = direction.getYaw();
            String modelId = "%s:block/%s".formatted(key.method_12836(), key.method_12832());

            int snappedY;
            if (y >= 315 || y < 45) {
                snappedY = 0;
            } else if (y < 135) {
                snappedY = 90;
            } else if (y < 225) {
                snappedY = 180;
            } else {
                snappedY = 270;
            }

            element.addProperty("model", modelId);
            if (snappedY != 0) {
                element.addProperty("y", snappedY);
            }

            String keyName = "facing_16=" + direction.method_15434();
            variants.add(keyName, element);
        }

        object.add("variants", variants);
        return object;
    }

    JsonElement buildBlockItem(class_2248 block) {
        class_2960 blockKey = class_7923.field_41175.method_10221(block);
        JsonObject object = new JsonObject();
        JsonObject model = new JsonObject();
        model.addProperty("type", "minecraft:model");
        model.addProperty("model", "%s:block/%s".formatted(blockKey.method_12836(), blockKey.method_12832()));
        object.add("model", model);
        return object;
    }
}

==> reverie_dreams-0.4.8%2B1.21.7%2Bbeta-sources.jar: cc/thonly/reverie_dreams/datagen/JsonElementWriterProvider.java <==
package cc.thonly.reverie_dreams.datagen;

import cc.thonly.reverie_dreams.block.base.FruitLeavesBlock;
import cc.thonly.reverie_dreams.data.FumoType;
import cc.thonly.reverie_dreams.datagen.generator.AbstractJsonElementWriterProvider;
import cc.thonly.reverie_dreams.item.base.SpawnEggItem;
import cc.thonly.reverie_dreams.registry.content.FumoTypes;
import cc.thonly.reverie_dreams.registry.content.block.RDBlocks;
import cc.thonly.reverie_dreams.registry.content.block.RDWoodBlocks;
import cc.thonly.reverie_dreams.state.SixteenDirection;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import net.fabricmc.fabric.api.datagen.v1.FabricDataOutput;
import net.minecraft.class_1792;
import net.minecraft.class_2248;
import net.minecraft.class_2397;
import net.minecraft.class_2960;
import net.minecraft.class_5321;
import net.minecraft.class_7225;
import net.minecraft.class_7923;
import java.util.concurrent.CompletableFuture;

public class JsonElementWriterProvider extends AbstractJsonElementWriterProvider {
    public JsonElementWriterProvider(FabricDataOutput output, CompletableFuture<class_7225.class_7874> future) {
        super(output, future);
    }

    @SuppressWarnings("deprecation")
    @Override
    protected void configured() {
        for (FumoType fumoType : FumoTypes.getView()) {
            class_2248 block = fumoType.block();
            this.addSixteenDirectionBlockState(block);
        }
        this.addSixteenDirectionBlockState(RDBlocks.MARISA_HAT_BLOCK);
        this.addFruityLeavesBlockState(RDWoodBlocks.LEMON_FRUIT_LEAVES, RDWoodBlocks.LEMON.leaves());
        this.addFruityLeavesBlockState(RDWoodBlocks.GINKGO_FRUIT_LEAVES, RDWoodBlocks.GINKGO.leaves());
        this.addFruityLeavesBlockState(RDWoodBlocks.PEACH_FRUIT_LEAVES, RDWoodBlocks.PEACH.leaves());

        this.addElement(Type.ASSETS, RDWoodBlocks.BLESSED_SPIRITUAL_LOG.method_40142().method_40237().method_29177(), "blockstates", strToJson(
                "{\n" +
                        "  \"variants\": {\n" +
                        "    \"axis=x\": {\n" +
                        "      \"model\": \"reverie_dreams:block/blessed_spiritual_log\",\n" +
                        "      \"x\": 90,\n" +
                        "      \"y\": 90\n" +
                        "    },\n" +
                        "    \"axis=y\": {\n" +
                        "      \"model\": \"reverie_dreams:block/blessed_spiritual_log\"\n" +
                        "    },\n" +
                        "    \"axis=z\": {\n" +
                        "      \"model\": \"reverie_dreams:block/blessed_spiritual_log\",\n" +
                        "      \"x\": 90\n" +
                        "    }\n" +
                        "  }\n" +
                        "}"
        ));
        this.addElement(Type.ASSETS, RDWoodBlocks.BLESSED_SPIRITUAL_LOG.method_40142().method_40237().method_29177(), "items", strToJson(
                "{\n" +
                        "  \"model\": {\n" +
                        "    \"type\": \"minecraft:model\",\n" +
                        "    \"model\": \"reverie_dreams:block/blessed_spiritual_log\"\n" +
                        "  }\n" +
                        "}"
        ));
        for (class_1792 spawnEgg : SpawnEggItem.SPAWN_EGGS) {
            class_5321<class_1792> key = spawnEgg.method_40131().method_40237();
            class_2960 location = key.method_29177();
            JsonElement element = strToJson("{\n" +
                    "  \"model\": {\n" +
                    "    \"type\": \"minecraft:condition\",\n" +
                    "    \"component\": \"minecraft:dyed_color\",\n" +
                    "    \"on_false\": {\n" +
                    "      \"type\": \"minecraft:model\",\n" +
                    "      \"model\": \"reverie_dreams:item/spawn_egg\"\n" +
                    "    },\n" +
                    "    \"on_true\": {\n" +
                    "      \"type\": \"minecraft:model\",\n" +
                    "      \"model\": \"reverie_dreams:item/spawn_egg_dyed\",\n" +
                    "      \"tints\": [\n" +
                    "        {\n" +
                    "          \"type\": \"minecraft:constant\",\n" +
                    "          \"value\": -1\n" +
                    "        },\n" +
                    "        {\n" +
                    "          \"type\": \"minecraft:dye\",\n" +
                    "          \"default\": 0\n" +
                    "        }\n" +
                    "      ]\n" +
                    "    },\n" +
                    "    \"property\": \"minecraft:has_component\"\n" +
                    "  }\n" +
                    "}");
            this.addElement(Type.ASSETS, location, "items", element);
        }
    }

    void addSixteenDirectionBlockState(class_2248 block) {
        class_2960 key = class_7923.field_41175.method_10221(block);
        JsonElement blockState = this.buildSixteenDirectionBlockState(block);
        JsonElement items = this.buildBlockItem(block);
        this.addElement(Type.ASSETS, key, "blockstates", blockState);
        this.addElement(Type.ASSETS, key, "items", items);
    }

    void addFruityLeavesBlockState(class_2248 block, class_2248 rawBlock) {
        class_2960 key = class_7923.field_41175.method_10221(block);
        JsonElement blockState = this.buildFruityLeavesBlock(block, rawBlock);
        JsonElement items = this.buildBlockItem(block);
        this.addElement(Type.ASSETS, key, "blockstates", blockState);
        this.addElement(Type.ASSETS, key, "items", items);
    }

    JsonElement buildFruityLeavesBlock(class_2248 block, class_2248 rawBlock) {
        class_2960 blockKey = class_7923.field_41175.method_10221(block);
        class_2960 rawBlockKey = class_7923.field_41175.method_10221(rawBlock);
        JsonObject object = new JsonObject();
        JsonObject variants = new JsonObject();
        for (int i = 0; i <= FruitLeavesBlock.MAX_AGE; i++) {
            for (Integer distance : class_2397.field_11199.method_11898()) {
                for (Boolean persistent : class_2397.field_11200.method_11898()) {
                    for (Boolean waterlogged : class_2397.field_38227.method_11898()) {
                        JsonObject element = new JsonObject();
                        String modelId = i == FruitLeavesBlock.MAX_AGE ? "%s:block/%s".formatted(blockKey.method_12836(), blockKey.method_12832()) : "%s:block/%s".formatted(rawBlockKey.method_12836(), rawBlockKey.method_12832());
                        element.addProperty("model", modelId);
                        String keyName = "fruit_age=%s,distance=%s,persistent=%s,waterlogged=%s".formatted(i, distance, persistent, waterlogged);
                        variants.add(keyName, element);
                    }
                }

            }
        }
        object.add("variants", variants);
        return object;
    }

    JsonElement buildSixteenDirectionBlockState(class_2248 block) {
        class_2960 key = class_7923.field_41175.method_10221(block);
        JsonObject object = new JsonObject();
        JsonObject variants = new JsonObject();

        for (SixteenDirection direction : SixteenDirection.values()) {
            JsonObject element = new JsonObject();
            float y = direction.getYaw();
            String modelId = "%s:block/%s".formatted(key.method_12836(), key.method_12832());

            int snappedY;
            if (y >= 315 || y < 45) {
                snappedY = 0;
            } else if (y < 135) {
                snappedY = 90;
            } else if (y < 225) {
                snappedY = 180;
            } else {
                snappedY = 270;
            }

            element.addProperty("model", modelId);
            if (snappedY != 0) {
                element.addProperty("y", snappedY);
            }

            String keyName = "facing_16=" + direction.method_15434();
            variants.add(keyName, element);
        }

        object.add("variants", variants);
        return object;
    }

    JsonElement buildBlockItem(class_2248 block) {
        class_2960 blockKey = class_7923.field_41175.method_10221(block);
        JsonObject object = new JsonObject();
        JsonObject model = new JsonObject();
        model.addProperty("type", "minecraft:model");
        model.addProperty("model", "%s:block/%s".formatted(blockKey.method_12836(), blockKey.method_12832()));
        object.add("model", model);
        return object;
    }
}
